---------------------------------------------------------------------------------
-- Navision Receipts
---------------------------------------------------------------------------------

drop function nav_import_receipt ();
create or replace function nav_import_receipt ()
returns integer as $body$
DECLARE
        row			RECORD;
	v_counter		integer;
	v_exists_p		integer;
	v_cost_id		integer;
	v_customer_id		integer;
	v_vendor_id		integer;
	v_project_id		integer;
	v_cost_type_id		integer;
	v_cost_status_id	integer;
	v_payment_days		integer;
	v_creation_user_id	integer;
	v_company_contact_id	integer;
	v_cost_center_id	integer;
BEGIN
    v_counter := 0;

    FOR row IN
        select	*
	from	nav."IT$Purch_ Rcpt_ Header"
	order by "No_"
    LOOP
	v_counter := v_counter + 1;

	------------------------ Custome and Provider  ---------------------------------------------
	select company_id into v_vendor_id from im_companies
	where nav_vendor_nr = row."Pay-to Vendor No_";
	IF v_vendor_id is null THEN
	   	RAISE WARNING 'Vendor % not found', row."Pay-to Vendor No_";
		continue;
	END IF;

	select company_id into v_customer_id from im_companies
	where company_path = 'internal';

	------------------------ Default Values ---------------------------------------------
	RAISE NOTICE 'Processing Receipt %: %', v_counter, row."No_";

	v_cost_status_id := import_cat('Created', 'Intranet Cost Status');
	v_cost_type_id := import_cat('Provider Receipt', 'Intranet Cost Type');

	-- Creation user
	select person_id into v_creation_user_id from persons
	where nav_windows_login = lower(row."User ID");

	-- Contact provider
	select person_id into v_company_contact_id from persons
	where lower(nav_contact_nr) ~ lower(row."Pay-to Contact No_");

	-- Project
	select project_id into v_project_id from im_projects
	where nav_project_nr = row."Project No_";

	-- Payment Conditions
	v_payment_days := NULL;
	IF row."Payment Terms Code" = '10_D' THEN v_payment_days := 10; END IF;
	IF row."Payment Terms Code" = '14_D' THEN v_payment_days := 14; END IF;
	IF row."Payment Terms Code" = '14_D_D' THEN v_payment_days := 14; END IF;
	IF row."Payment Terms Code" = '20_D' THEN v_payment_days := 20; END IF;
	IF row."Payment Terms Code" = '21_D' THEN v_payment_days := 21; END IF;
	IF row."Payment Terms Code" = '30_D' THEN v_payment_days := 30; END IF;
	IF row."Payment Terms Code" = '30_D_D' THEN v_payment_days := 30; END IF;
	IF row."Payment Terms Code" = '45_D' THEN v_payment_days := 45; END IF;
	IF row."Payment Terms Code" = '45_D_D' THEN v_payment_days := 45; END IF;
	IF row."Payment Terms Code" = '60_D' THEN v_payment_days := 60; END IF;
	IF row."Payment Terms Code" = '60_D_D' THEN v_payment_days := 60; END IF;
	IF row."Payment Terms Code" = '75_D' THEN v_payment_days := 75; END IF;
	IF row."Payment Terms Code" = '90_D' THEN v_payment_days := 90; END IF;
	IF row."Payment Terms Code" = 'HALF_YEAR' THEN v_payment_days := 182; END IF;
	IF row."Payment Terms Code" = '2_Y' THEN v_payment_days := 730; END IF;
	IF row."Payment Terms Code" = 'IMM' THEN v_payment_days := 0; END IF;
	IF row."Payment Terms Code" = 'UNDO' THEN v_payment_days := 10; END IF;
	IF row."Payment Terms Code" = 'NEPLATIT' THEN v_payment_days := 10000; END IF;

	-- make sure the currency code exists
	select count(*) into v_exists_p from currency_codes
	where iso = row."Currency Code";
	IF v_exists_p = 0 THEN
		RAISE WARNING 'Currency "%" does not exist for FinDoc: %', row."Currency Code", row."No_";
		continue;
	END IF;

	------------------------ New Cost and Invoice ------------------------------------

	-- Check if the cost item is already there
	select cost_id into v_cost_id from im_costs c
	where c.cost_name = row."No_";
	IF v_cost_id is null THEN
	   	RAISE NOTICE 'Insert Invoice: %', row."No_";
		v_cost_id := im_invoice__new (
			null, 'im_invoice', row."Document Date"::timestamptz, v_creation_user_id, '0.0.0.0', null,

			row."No_",			-- cost_nr
			v_customer_id,			-- customer
			v_vendor_id,			-- provider
			null,				-- customer contact
			row."Order Date",		-- effective_date 

			row."Currency Code",		-- currency
			null,				-- template_id
			v_cost_status_id,		-- status_id
			v_cost_type_id,			-- type_id

			null,				-- payment_method_id
			v_payment_days,			-- payment_days
			0.0,				-- amount
			null,				-- vat
			null,				-- tax
			null				-- note
		);
	END IF;

	update im_costs set
		cost_name		= row."No_",
		cost_nr			= v_cost_id::varchar,
		customer_id		= v_customer_id,
		provider_id		= v_vendor_id,
		project_id		= v_project_id,
		effective_date		= row."Order Date",
		currency		= row."Currency Code",
		cost_status_id		= v_cost_status_id,
		cost_type_id		= v_cost_type_id,
		payment_days		= v_payment_days,
		amount			= 0.0,
		cost_center_id		= v_cost_center_id,
		tax			= null,
		vat			= null,
		nav_purchase_receipt_nr = row."No_"
	where cost_id = v_cost_id;

	update im_invoices set
			company_contact_id = v_company_contact_id,
			invoice_nr = row."No_",
			payment_method_id = null
	where invoice_id = v_cost_id;

    END LOOP;
    RETURN 0;
END;$body$ language 'plpgsql';
select nav_import_receipt ();




drop function nav_import_receipt_lines ();
create or replace function nav_import_receipt_lines ()
returns integer as $body$
DECLARE
        row			RECORD;
	v_counter		integer;
	v_cost_id		integer;
	v_exists_p		integer;
	v_name			varchar;
	v_uom_id		integer;
	v_line_id		integer;
	v_currency_code		varchar;
	v_material_id		integer;
	v_amount		numeric;
BEGIN
    v_counter := 0;

    FOR row IN
        select	*
	from	nav."IT$Purch_ Rcpt_ Line"
	where	"Document No_" in (select cost_name from im_costs)
	order by "Document No_", "Line No_"
    LOOP
	v_counter := v_counter + 1;

	------------------------ Financial Document  ---------------------------------------------
	select cost_id into v_cost_id from im_costs
	where cost_name = row."Document No_";
	IF v_cost_id is null THEN
	   	RAISE WARNING 'FinDoc % not found', row."Document No_";
		continue;
	END IF;

	select currency into v_currency_code from im_costs 
	where cost_id = v_cost_id;

	------------------------ Transform Fields ------------------------------------

	-- Unit of Measure
	v_uom_id := import_cat ('Unit', 'Intranet UoM');
	IF row."Unit of Measure" = 'hodina' THEN v_uom_id = import_cat ('Hour', 'Intranet UoM'); END IF;
	IF row."Unit of Measure" = 'slovo' THEN v_uom_id = import_cat ('S-Word', 'Intranet UoM'); END IF;
	IF row."Unit of Measure" = 'ks' THEN v_uom_id = import_cat ('Unit', 'Intranet UoM'); END IF;
	IF row."Unit of Measure" = 'den' THEN v_uom_id = import_cat ('Day', 'Intranet UoM'); END IF;
	IF row."Unit of Measure" = 'NS' THEN v_uom_id = import_cat ('Page', 'Intranet UoM'); END IF;
	IF row."Unit of Measure" = 'strana' THEN v_uom_id = import_cat ('Page', 'Intranet UoM'); END IF;
	IF row."Unit of Measure" = 'iNEW WORD' THEN v_uom_id = import_cat ('S-Word', 'Intranet UoM'); END IF;
	IF row."Unit of Measure" = 'license' THEN v_uom_id = import_cat ('Unit', 'Intranet UoM'); END IF;
	IF row."Unit of Measure" = 'znak' THEN v_uom_id = import_cat ('Unit', 'Intranet UoM'); END IF;
	IF row."Unit of Measure" = 'kus' THEN v_uom_id = import_cat ('Unit', 'Intranet UoM'); END IF;

	-- Name Generation
	-- Names need to be unique in ]po[
	v_name := coalesce(row."Description 2", row."Description");
	v_name := v_name || ' (' || row."Source Language" || ' -> ' || row."Target Language" || ')';

	-- Material
	select material_id into v_material_id from im_materials
	where material_nr = 'default';

	------------------------ New Line ------------------------------------

	-- Check for duplicated entries (i.e. names)
	select	count(*) into v_exists_p 
	from	im_invoice_items
	where	item_name = v_name and
		invoice_id = v_cost_id and
		sort_order = row."Line No_" and
		item_uom_id = v_uom_id;

	IF v_exists_p > 1 THEN
		RAISE WARNING 'Found more then one invoice line matching criteria for FinDoc: %', row."Document No_";
		continue;
	END IF;

	select	item_id into v_line_id
	from	im_invoice_items
	where	item_name = v_name and
		invoice_id = v_cost_id and
		sort_order = row."Line No_" and
		item_uom_id = v_uom_id;

	IF v_line_id is null THEN
	   	RAISE NOTICE 'Insert Invoice Line for: %', row."Document No_";
		v_line_id := nextval('im_invoice_items_seq');
		insert into im_invoice_items (
			item_id,
			item_name,
			invoice_id,
			item_units,
			item_uom_id,
			price_per_unit,
			currency,
			sort_order,
			item_type_id,
			item_status_id,
			description,
			item_material_id
		) values (
			v_line_id,
			v_name,
			v_cost_id,
			row."Quantity",
			v_uom_id,
			row."Direct Unit Cost",
			v_currency_code,
			row."Line No_",
			null,
			null,
			null,
			v_material_id
		);
	ELSE
	   	RAISE NOTICE 'Update Invoice Line for: %', row."Document No_";
	END IF;

	update im_invoice_items set
			item_name		= v_name,
			invoice_id		= v_cost_id,
			item_units		= row."Quantity",
			item_uom_id		= v_uom_id,
			price_per_unit		= row."Direct Unit Cost",
			currency		= v_currency_code,
			sort_order		= row."Line No_",
			item_type_id		= null,
			item_status_id		= null,
			description		= null,
			item_material_id	= v_material_id
	where item_id = v_line_id;

	-- Update the amount of the invoice
	select amount into v_amount from im_costs
	where cost_id = v_cost_id;
	v_amount := v_amount + row."Direct Unit Cost" * row."Quantity";
	update im_costs set amount = v_amount
	where cost_id = v_cost_id;

    END LOOP;
    RETURN 0;
END;$body$ language 'plpgsql';
select nav_import_receipt_lines ();



