---------------------------------------------------------------------------------
-- Navision Vendors
---------------------------------------------------------------------------------

drop function nav_import_vendor ();
create or replace function nav_import_vendor ()
returns integer as $body$
DECLARE
        row			RECORD;
	v_company_id		integer;
	v_office_id		integer;
	v_exists_p		integer;
	v_company_name		varchar;
	v_office_name		varchar;
	v_office_status_id	integer;
	v_office_type_id	integer;
	v_company_status_id	integer;
	v_company_type_id	integer;
	v_duplicate_p		integer;
	v_payment_days		integer;
	v_primary_contact_id	integer;
	v_note			varchar;
	v_parent_company_id	integer;
	v_nav_pm_id		integer;
	v_nav_account_manager_id integer;
	v_country_code		varchar;
	v_counter		integer;
BEGIN
    v_counter := 0;

    FOR row IN
        select	*
	from	nav."Vendor"
	order by "No_"
    LOOP
	v_counter := v_counter + 1;

	------------------------ Name Fiddling ---------------------------------------------
	-- Process the name in order to deal with duplicates.
	-- ]po[ doesnt allow for duplicate company or office names.

	-- By default just use the standard name
	v_company_name := row."Name";

	-- Add the vendor no_ if the name is not unique within the NAV table
	select	count(*) into v_duplicate_p from nav."Vendor"
	where	lower("Name") = lower(v_company_name);
	IF v_duplicate_p > 1 THEN
		v_company_name := v_company_name || ' (' || row."No_" || ')';
	END IF;


	-- The office name is the company name + "Main Office".
	v_office_name := v_company_name || ' Main Office';


	------------------------ Default Values ---------------------------------------------
	RAISE NOTICE 'Processing Vendor %: %', v_counter, v_company_name;

	-- Default Status for company and office
	v_office_status_id := import_cat('Active', 'Intranet Office Status');
	v_office_type_id := import_cat('Main Office', 'Intranet Office Type');
	v_company_status_id := import_cat('Active', 'Intranet Company Status');
	v_company_type_id := import_cat('Provider', 'Intranet Company Type');

	------------------------ New Office and Comany ------------------------------------
	-- Check if the office is already there
	select	office_id into v_office_id from im_offices o
	where	lower(o.office_name) = lower(v_office_name);
	IF v_office_id is null THEN
		select office_id into v_office_id from im_offices o
		where o.nav_vendor_nr = row."No_";
	END IF;

	-------------------------------------------------------------------------------------
	-- Fix Country Codes
	v_country_code := lower(row."Country_Region Code");
	IF v_country_code = 'gb' THEN v_country_code := 'uk'; END IF;
	IF v_country_code = '' THEN v_country_code := NULL; END IF;

	select count(*) into v_exists_p
	from country_codes where iso = v_country_code;
	IF v_country_code is not null and v_exists_p < 1 THEN
	   	RAISE WARNING 'Skipping (unknown country code %): %', v_country_code, v_office_name;
		continue;
	END IF;

	-------------------------------------------------------------------------------------
	-- Create a new office first, because the im_company.main_office_id is not null
	IF v_office_id is null THEN
	   	RAISE NOTICE 'Insert Office: %', v_office_name;

		select count(*) into v_exists_p from im_offices
		where lower(office_name) = lower(v_office_name);
		IF v_exists_p > 0 THEN
			v_office_name := v_office_name || ' (' || row."No_" || ')';
		END IF;

		v_office_id := im_office__new (
			NULL, 'im_office', now()::date, 0, '0.0.0.0', null,
			v_office_name, row."No_", v_office_type_id, 
			v_office_status_id, null
		);
		update im_offices set nav_vendor_nr = row."No_" where office_id = v_office_id;
	END IF;

	-- Check if the company is already there
	select	company_id into v_company_id from im_companies c
	where	c.nav_vendor_nr = row."No_";

	IF v_company_id is null THEN
	   	RAISE NOTICE 'Insert Company: %', v_company_name;

		select count(*) into v_exists_p from im_companies
		where lower(company_name) = lower(v_company_name);
		IF v_exists_p > 0 THEN
			v_company_name := v_company_name || ' (' || row."No_" || ')';
		END IF;

		v_company_id := im_company__new (
			NULL, 'im_company', now()::date, 0, '0.0.0.0', null,
			v_company_name, row."No_",
			v_office_id, v_company_type_id, v_company_status_id
		);
		update im_companies set nav_vendor_nr = row."No_" where company_id = v_company_id;
	END IF;

	------------------------ Update Office and Comany----------------------------------
	
	-- Conver Payment Terms Code
	v_payment_days := NULL;
	IF row."Payment Terms Code" = '10_D' THEN v_payment_days := 10; END IF;
	IF row."Payment Terms Code" = '14_D' THEN v_payment_days := 14; END IF;
	IF row."Payment Terms Code" = '14_D_D' THEN v_payment_days := 14; END IF;
	IF row."Payment Terms Code" = '20_D' THEN v_payment_days := 20; END IF;
	IF row."Payment Terms Code" = '21_D' THEN v_payment_days := 21; END IF;
	IF row."Payment Terms Code" = '30_D' THEN v_payment_days := 30; END IF;
	IF row."Payment Terms Code" = '30_D_D' THEN v_payment_days := 30; END IF;
	IF row."Payment Terms Code" = '45_D' THEN v_payment_days := 45; END IF;
	IF row."Payment Terms Code" = '45_D_D' THEN v_payment_days := 45; END IF;
	IF row."Payment Terms Code" = '60_D' THEN v_payment_days := 60; END IF;
	IF row."Payment Terms Code" = '60_D_D' THEN v_payment_days := 60; END IF;
	IF row."Payment Terms Code" = '75_D' THEN v_payment_days := 75; END IF;
	IF row."Payment Terms Code" = '90_D' THEN v_payment_days := 90; END IF;
	IF row."Payment Terms Code" = 'HALF_YEAR' THEN v_payment_days := 182; END IF;
	IF row."Payment Terms Code" = '2_Y' THEN v_payment_days := 730; END IF;
	IF row."Payment Terms Code" = 'IMM' THEN v_payment_days := 0; END IF;
	IF row."Payment Terms Code" = 'UNDO' THEN v_payment_days := 10; END IF;
	IF row."Payment Terms Code" = 'NEPLATIT' THEN v_payment_days := 10000; END IF;


	-- Add other fields to the "note" field
	v_note := NULL;
	IF row."Home Page" is not null AND row."Home Page" <> '' THEN v_note := v_note || 'Home Page=' || row."Home Page" ||', '; END IF;

	IF row."VAT Bus_ Posting Group" is not null AND row."VAT Bus_ Posting Group" <> '' THEN v_note := v_note || 'VAT Bus Posting Group=' || row."VAT Bus_ Posting Group" || ', '; END IF;
	IF row."Registration No_" is not null AND row."Registration No_" <> '' THEN v_note := v_note || 'Registration No_=' || row."Registration No_" ||', '; END IF;
	IF row."Contact Text" is not null AND row."Contact Text" <> '' THEN v_note := v_note || 'Contact Text=' || row."Contact Text" ||', '; END IF;
	IF row."Bank Account No_" is not null AND row."Bank Account No_" <> '' THEN v_note := v_note || 'Bank Account No_=' || row."Bank Account No_" ||', '; END IF;


	-- Primary contact
	select	person_id into v_primary_contact_id from persons
	where	nav_contact_nr ~ row."Primary Contact No_";
	IF v_primary_contact_id is null THEN
		IF row."E-Mail" is not null AND row."E-Mail" <> '' THEN
			v_primary_contact_id := lookup_user_email(row."E-Mail", 'nav_import_vendor.im_company.primary_contact_id');
		END IF;
	END IF;

	-- Default Project Manager
	select	person_id into v_nav_pm_id from persons
	where	nav_employee_nr = row."Project Manager";

	-- Parent Company
	select	company_id into v_parent_company_id from im_companies
	where	nav_vendor_nr = row."Mother Company";

	RAISE NOTICE 'Update Office: %', v_office_name;
	update im_offices set
			office_name = v_office_name,
			office_status_id = v_office_status_id,
			office_type_id = v_office_type_id,
        		phone = row."Phone No_",
			fax = row."Fax No_",
        		address_line1 = row."Address",
        		address_line2 = row."Address 2",
        		address_city = row."City",
        		address_country_code = v_country_code,
        		address_postal_code = row."Post Code",
			nav_vendor_nr = row."No_"
	where office_id = v_office_id;

	RAISE NOTICE 'Update Company: %', v_company_name;
	update im_companies set
			company_name = v_company_name,
			main_office_id = v_office_id,
			company_status_id = v_company_status_id,
			company_type_id = v_company_type_id,
			vat_number = row."VAT Registration No_",
			default_payment_days = v_payment_days,
			primary_contact_id = v_primary_contact_id,
			accounting_contact_id = v_primary_contact_id,
			parent_company_id = v_parent_company_id,
        		note = v_note,
			nav_vendor_posting_group = row."Vendor Posting Group",
			nav_pm_percentage = row."PM %",
			nav_vendor_nr = row."No_"
	where company_id = v_company_id;

	-- Account Manager: Establish a im_biz_object_member relationship
	-- between the company and the account manager with as "Key Account".
	select	person_id into v_nav_account_manager_id from persons
	where	nav_employee_nr = row."Account Manager";
	IF v_nav_account_manager_id is not null THEN
		RAISE NOTICE 'Adding Account Manager for company: %', v_company_name;
		PERFORM im_biz_object_member__new (
			null,
			'im_biz_object_member',
			v_company_id,			-- the business object
			v_nav_account_manager_id,	-- the person related to biz_object
			1302,				-- role: Key Account
			null,				-- percentage
			null,				-- creation user
			'0.0.0.0'			-- creation IP
		);
	END IF;

    END LOOP;
    RETURN 0;
END;$body$ language 'plpgsql';
select nav_import_vendor ();



