--------------------------------------------------------------------------------
-- Views
---------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION sa.ms_getdate()
RETURNS timestamp without time zone AS 'SELECT LOCALTIMESTAMP'
LANGUAGE 'sql' VOLATILE;


CREATE or replace VIEW sa.view_ar_relationshipall_parent_valid AS
SELECT 
	PARENT."ObjectID",
	PARENT."ObjectName",
	PARENT."ObjectAlias",
	PARENT."ObjectDescription",
	PARENT."ObjectTypeID",
	PARENT."ObjectCustomField1",
	PARENT."ObjectCustomField2",
	PARENT."ObjectValidFrom",
	PARENT."ObjectValidTo",
	PARENT_OBJTP."ObjectTypeNamespace",
	PARENT_OBJTP."ObjectTypeAlias",
	deriv_parent."ParentObjectID",
	deriv_parent."ChildObjectID",
	deriv_parent."RelationshipValidFrom",
	deriv_parent."RelationshipValidTo",
	deriv_parent."TransitivePath",
	PARENT."OwnerID"
FROM
	(SELECT  "ParentObjectID",
		"ChildObjectID",
		MIN(DERIV_TBL."RelationshipValidTo") As "RelationshipValidTo",
		MAX(DERIV_TBL."RelationshipValidFrom") As "RelationshipValidFrom",
		"TransitivePath"
	FROM
		(SELECT sa."AR_Relationship"."ParentObjectID",
			sa."AR_Relationship"."ChildObjectID",
			sa."AR_Relationship"."RelationshipValidFrom",
			sa."AR_Relationship"."RelationshipValidTo",
			CAST(sa."AR_Relationship"."ParentObjectID" AS varchar) || ',' || 
				CAST(sa."AR_Relationship"."ChildObjectID" AS varchar) || ',' AS "TransitivePath"
		FROM 	  sa."AR_Relationship"
		WHERE     "RelationshipValidFrom" <=sa.ms_getdate() AND
		          (("RelationshipValidTo" >= sa.ms_getdate()) OR ("RelationshipValidTo" is NULL))
		UNION
		SELECT  sa."AR_RelationshipAll"."ParentObjectID",
			sa."AR_RelationshipAll"."ChildObjectID",
			sa."AR_RelationshipAll"."RelationshipValidFrom", 
			sa."AR_RelationshipAll"."RelationshipValidTo",
			sa."AR_RelationshipAll"."TransitivePath"
		FROM    sa."AR_RelationshipAll"
		WHERE      (sa."AR_RelationshipAll"."RelationshipValidFrom" <= sa.ms_getdate()) AND
	        	  ((sa."AR_RelationshipAll"."RelationshipValidTo" >= sa.ms_getdate()) 
				OR (sa."AR_RelationshipAll"."RelationshipValidTo" is NULL))
		UNION
		SELECT	sa."AR_RelationshipAll"."ParentObjectID", 
			sa."AR_Relationship"."ChildObjectID", 
			sa."AR_RelationshipAll"."RelationshipValidFrom", 
			sa."AR_RelationshipAll"."RelationshipValidTo",
			sa."AR_RelationshipAll"."TransitivePath" || 
				CAST(sa."AR_Relationship"."ChildObjectID" AS varchar) || ', ' AS "TransitivePath"
		FROM	sa."AR_Relationship"
			INNER JOIN sa."AR_RelationshipAll" 
				ON sa."AR_Relationship"."ParentObjectID" = sa."AR_RelationshipAll"."ChildObjectID"
		WHERE	sa."AR_Relationship"."RelationshipValidFrom" <= sa.ms_getdate() AND
			((sa."AR_Relationship"."RelationshipValidTo" >= sa.ms_getdate()) OR 
				(sa."AR_Relationship"."RelationshipValidTo" is NULL)) AND
			sa."AR_RelationshipAll"."RelationshipValidFrom" <= sa.ms_getdate() AND
			((sa."AR_RelationshipAll"."RelationshipValidTo" >= sa.ms_getdate()) 
				OR (sa."AR_RelationshipAll"."RelationshipValidTo" is NULL))
		) AS DERIV_TBL
	GROUP BY
		"ParentObjectID", 
		"ChildObjectID", 
		"TransitivePath" 
	) AS deriv_parent
	LEFT JOIN sa."AR_Object" PARENT on PARENT."ObjectID" = deriv_parent."ParentObjectID"
	LEFT OUTER JOIN sa."AR_ObjectType" PARENT_OBJTP ON PARENT_OBJTP."ObjectTypeID" = PARENT."ObjectTypeID";




CREATE OR REPLACE VIEW sa.view_ar_relationshipall_child_valid AS
select
	CHILD."ObjectID",CHILD."ObjectName",CHILD."ObjectAlias",CHILD."ObjectDescription",CHILD."ObjectTypeID",
	CHILD."ObjectCustomField1",CHILD."ObjectCustomField2",CHILD."ObjectValidFrom",CHILD."ObjectValidTo",
	CHILD_OBJTP."ObjectTypeNamespace",CHILD_OBJTP."ObjectTypeAlias", deriv_child."ParentObjectID", 
	deriv_child."ChildObjectID",deriv_child."RelationshipValidFrom", deriv_child."RelationshipValidTo", 
	deriv_child."TransitivePath", CHILD."OwnerID"
FROM
	(SELECT  "ParentObjectID", "ChildObjectID",
		MIN(DERIV_TBL."RelationshipValidTo") As "RelationshipValidTo", 
		MAX(DERIV_TBL."RelationshipValidFrom") As "RelationshipValidFrom",
		"TransitivePath"
	FROM
		(SELECT	sa."AR_Relationship"."ParentObjectID", sa."AR_Relationship"."ChildObjectID", 
			sa."AR_Relationship"."RelationshipValidFrom", sa."AR_Relationship"."RelationshipValidTo",
			CAST(sa."AR_Relationship"."ParentObjectID" As varchar) || ', ' || 
				CAST(sa."AR_Relationship"."ChildObjectID" As varchar) || ', ' As "TransitivePath"
		FROM 	  sa."AR_Relationship"
		WHERE     "RelationshipValidFrom" <=sa.ms_getdate() AND
		          (("RelationshipValidTo" >= sa.ms_getdate()) OR ("RelationshipValidTo" is NULL))
		UNION
		SELECT	sa."AR_RelationshipAll"."ParentObjectID", sa."AR_RelationshipAll"."ChildObjectID", 
			sa."AR_RelationshipAll"."RelationshipValidFrom", sa."AR_RelationshipAll"."RelationshipValidTo",
			sa."AR_RelationshipAll"."TransitivePath"
		FROM    sa."AR_RelationshipAll"
		WHERE   sa."AR_RelationshipAll"."RelationshipValidFrom" <=sa.ms_getdate() AND
			((sa."AR_RelationshipAll"."RelationshipValidTo" >= sa.ms_getdate()) OR 
				(sa."AR_RelationshipAll"."RelationshipValidTo" is NULL))
		UNION
		SELECT	sa."AR_RelationshipAll"."ParentObjectID", sa."AR_Relationship"."ChildObjectID", 
			sa."AR_RelationshipAll"."RelationshipValidFrom", sa."AR_RelationshipAll"."RelationshipValidTo",
			sa."AR_RelationshipAll"."TransitivePath" || CAST(sa."AR_Relationship"."ChildObjectID" As varchar) || 
				', ' As "TransitivePath"
		FROM	sa."AR_Relationship"
			INNER JOIN sa."AR_RelationshipAll" 
				ON sa."AR_Relationship"."ParentObjectID" = sa."AR_RelationshipAll"."ChildObjectID"
		WHERE	sa."AR_Relationship"."RelationshipValidFrom" <=sa.ms_getdate() AND
			((sa."AR_Relationship"."RelationshipValidTo" >= sa.ms_getdate()) OR 
				(sa."AR_Relationship"."RelationshipValidTo" is NULL)) AND
			sa."AR_RelationshipAll"."RelationshipValidFrom" <=sa.ms_getdate() AND
			((sa."AR_RelationshipAll"."RelationshipValidTo" >= sa.ms_getdate()) OR 
				(sa."AR_RelationshipAll"."RelationshipValidTo" is NULL))
		) AS DERIV_TBL
	GROUP BY "ParentObjectID", "ChildObjectID", "TransitivePath" 
	) AS deriv_child
	LEFT JOIN sa."AR_Object" CHILD on CHILD."ObjectID" = deriv_child."ChildObjectID"
	LEFT OUTER JOIN sa."AR_ObjectType" CHILD_OBJTP ON CHILD_OBJTP."ObjectTypeID" = CHILD."ObjectTypeID";



---------------------------------------------------------------------------------
-- Secure Access Mapping from AR_Object to groups
---------------------------------------------------------------------------------

drop table sa_group_mapping;
create table sa_group_mapping (
	sa_group_name	    text,
	po_group_name	    text
);

-- Senior Managers
insert into sa_group_mapping VALUES ('MBU_LGMs','Senior Managers');
insert into sa_group_mapping VALUES ('Acn_Loc_LGMs','Senior Managers');
insert into sa_group_mapping VALUES ('Acn_Loc_LGM','Senior Managers');
insert into sa_group_mapping VALUES ('LGM','Senior Managers');
insert into sa_group_mapping VALUES ('MBU_LGMs','Senior Managers');
insert into sa_group_mapping VALUES ('Loc_LGMs','Senior Managers');

-- Accounting
insert into sa_group_mapping VALUES ('Accounting','Accounting');

-- Project Managers
insert into sa_group_mapping VALUES ('Acn_PM_PMs','Project Managers');
insert into sa_group_mapping VALUES ('Acn_PM_Pms','Project Managers');
insert into sa_group_mapping VALUES ('Acn_Ling_PMs','Project Managers');
insert into sa_group_mapping VALUES ('Acn_MBU_PMs','Project Managers');
insert into sa_group_mapping VALUES ('PMs_LU','Project Managers');
insert into sa_group_mapping VALUES ('PMs','Project Managers');

-- Project Coordinators
insert into sa_group_mapping VALUES ('Acn_DTP_Leads','Project Coordinators');
insert into sa_group_mapping VALUES ('Acn_Ling_Leads','Project Coordinators');
insert into sa_group_mapping VALUES ('Acn_LocEng_Leads','Project Coordinators');
insert into sa_group_mapping VALUES ('Acn_DTP_Leads','Project Coordinators');
insert into sa_group_mapping VALUES ('MBU_Leads','Project Coordinators');
insert into sa_group_mapping VALUES ('DTP_Leads','Project Coordinators');
insert into sa_group_mapping VALUES ('Loc_Leads','Project Coordinators');

-- HR Managers
insert into sa_group_mapping VALUES ('Acn_HR_Members','HR Managers');

-- Sales
insert into sa_group_mapping VALUES ('Acn_GBS_Mgmt','Sales');

-- Resource/Freelance Managers
insert into sa_group_mapping VALUES ('BPG_FRL','Freelance Managers');

-- Customers
insert into sa_group_mapping VALUES ('Acn_All_Customers','Customers');

-- Freelancers + Vendors
insert into sa_group_mapping VALUES ('Acn_All_Vendors','Freelancers');
insert into sa_group_mapping VALUES ('Acn_Freelancers_CZ','Freelancers');
insert into sa_group_mapping VALUES ('Acn_Freelancers_SK','Freelancers');
insert into sa_group_mapping VALUES ('Acn_BusinessPartnerPortal_Vendor','Freelancers');

-- Employees
insert into sa_group_mapping VALUES ('Developers','Employees');
insert into sa_group_mapping VALUES ('All.Hilleho','Employees');
insert into sa_group_mapping VALUES ('PMs','Employees');





---------------------------------------------------------------------------------
-- Secure Access Contacts
---------------------------------------------------------------------------------

-- Create new users by going through the ar_users table
-- line by line and inserting the user into the DB.
drop function sa_import_user();
drop function sa_import_user(integer, integer);
create or replace function sa_import_user (integer, integer)
returns integer as $BODY$
DECLARE
	p_offset		alias for $1;
	p_limit			alias for $2;

        row_user		RECORD;
        row_group		RECORD;

	v_loginname		varchar;
	v_uname			varchar;
	v_aname			varchar;
	v_authority_id		integer;
	v_old_authority_id	integer;
	v_old_authority_short_name varchar;
	v_update_authority_p	integer;
	v_count			integer;
	v_user_id		integer;
	v_duplicate_user_id	integer;
	v_exists_p		integer;
	v_username		varchar;
	v_group_id		integer;
	v_email			varchar;
	v_email_name		varchar;
	v_email_domain		varchar;
	v_sa_group_object_id	integer;
BEGIN
    v_count := 0;
    FOR row_user IN
        select
		*
	from
		sa."AR_User" u
		LEFT JOIN sa."AR_Object" o ON (u."ObjectID" = o."ObjectID")
	where
		1=1
	order by 
		u."ObjectID" DESC
	LIMIT	p_limit+2
	OFFSET	p_offset
    LOOP
	v_count := v_count + 1;

	-- get and massage the email.
	v_email := lower(row_user."EmailAddress");
	v_email_domain := substring(v_email from '@(.*)');
	v_email_name := substring(v_email from '(.*)@');
	v_email := v_email_name || '@' || v_email_domain;

	IF row_user."ObjectValidFrom" > now() OR row_user."ObjectValidTo" < now() THEN 
	   	RAISE NOTICE 'Skipping User (invalid from/to): % % (%)', row_user."FirstName", row_user."LastName", v_email;
		continue; 
	END IF;
	IF v_email is null or trim(v_email) = '' THEN 
	   	RAISE NOTICE 'Skipping User (empty email): % % (%)', row_user."FirstName", row_user."LastName", v_email;
		continue; 
	END IF;
	IF row_user."FirstName" is null OR row_user."FirstName" = '' THEN 
	   	RAISE NOTICE 'Skipping User (empty first name): % % (%)', row_user."FirstName", row_user."LastName", v_email;
		continue; 
	END IF;
	IF row_user."LastName" is null OR row_user."LastName" = '' THEN 
	   	RAISE NOTICE 'Skipping User (empty last name): % % (%)', row_user."FirstName", row_user."LastName", v_email;
		continue; 
	END IF;
	IF v_email is null OR v_email = '' THEN
		v_email := row_user."FirstName" || '.' || row_user."LastName" || '@nowhere.com';
	END IF;

	-- Split loginname into uname and aname (authority name)
	v_loginname := lower(row_user."Loginname");
	v_uname := substring(v_loginname from '\\\\(.*)');
	v_aname := substring(v_loginname from '(.*)\\\\');

	IF v_loginname like 'ex_' THEN
	   	RAISE NOTICE 'Skipping User (ex_*): % % (%) %', row_user."FirstName", row_user."LastName", v_email, v_loginname;
		continue; 
	END IF;

	-- Determine the authority
	select authority_id into v_authority_id from auth_authorities
	where short_name = v_aname;
	IF v_authority_id is null THEN 
	   	RAISE WARNING 'Authority not found: % % (%) (%)', row_user."FirstName", row_user."LastName", v_email, v_loginname;
		continue; 
	END IF;


	-- Check if the user already exists / has been imported already during 
	-- the last run of this script. PrimKey is the Secure Access ObjectID.
	v_user_id := null;
	select	min(party_id) into v_user_id from parties
	where	trim(lower(email)) = trim(lower(v_email));
	-- Email not found. Let's check for changed emails etc.
	IF v_user_id is null THEN
		select	person_id into v_user_id from persons
		where	sa_object_id = row_user."ObjectID";
	END IF;


	-- Skip the import if we would overwrite a non-mnet account
	select a.authority_id, a.short_name into v_old_authority_id, v_old_authority_short_name
	from auth_authorities a, users u 
	where u.authority_id = a.authority_id and u.user_id = v_user_id;

	IF v_old_authority_short_name is not null and v_old_authority_short_name != 'mnet' and v_old_authority_short_name != 'local' and v_aname = 'mnet' THEN 
	   	RAISE NOTICE 'Skipping User (do not overwrite a non-mnet account): % % (%) % vs. %', row_user."FirstName", row_user."LastName", v_email, v_loginname, v_old_authority_short_name;
		continue; 
	END IF;


	RAISE NOTICE 'Processing User #%: First=%, Last=%, Login=%, Email=%, OID=% => %', v_count, row_user."FirstName", row_user."LastName", v_loginname, v_email, row_user."ObjectID", v_user_id;

	IF v_user_id is null THEN
		-- Create a new user if the user wasnt there:
	   	RAISE NOTICE 'Insert User: % % (%)', row_user."FirstName", row_user."LastName", v_email;
		v_user_id := acs__add_user(
			null, 'user', now(), 0, '0.0.0.0', 
			null, v_uname, v_email, null,
			row_user."FirstName", row_user."LastName",
			'hashed_password', 'salt', 
			v_uname, 't', 'approved'
		);
		INSERT INTO users_contact (user_id) VALUES (v_user_id);
		INSERT INTO im_employees (employee_id) VALUES (v_user_id);
		INSERT INTO im_freelancers (user_id) VALUES (v_user_id);
	ELSE
		-- Check that the user does not have a duplicate auth_id/username
		-- (auth_id/username are not checked as "primary key", but they
		-- are unique).
		select	user_id into v_duplicate_user_id
		from	users u
		where	u.user_id != v_user_id and
			u.username = v_uname and
			u.authority_id = v_authority_id;

		IF v_duplicate_user_id is not null THEN
		   	RAISE WARNING 'Skipping User because of duplicate auth/username: % % (%). auth_id=%, username=%', row_user."FirstName", row_user."LastName", v_email, v_authority_id, v_uname;
			continue; 
		END IF;
	END IF;

	-- users_contact contains the address of a user.
	update users_contact set
			wa_line1 = row_user."Address1",
			wa_line2 = row_user."Address2" || row_user."Address2",
			wa_city = row_user."City",
			wa_postal_code = row_user."Zip",
			work_phone = row_user."PhoneOffice",
			fax = row_user."Fax"
	where user_id = v_user_id;

	update users set
			username = v_uname,
			authority_id = v_authority_id
	where user_id = v_user_id;

	update persons set
			sa_object_id = row_user."ObjectID",
			first_names = row_user."FirstName",
			last_name = row_user."LastName",
			nav_middle_name = row_user."MiddleName",
			nav_windows_login = substring(v_email from '^([^@]+)')
	where person_id = v_user_id;

	update im_employees set
			job_title = row_user."Title"
	where employee_id = v_user_id;


	-- Add the user to groups he is not yet a member of:
	-- Unfortunately, there may be multiple accounts for a
	-- single email, so we have to build the union of the
	-- group memberships of all the users accounts.

	-- Get the final email address of the guy:
	SELECT lower(email) into v_email from parties where party_id = v_user_id;

	FOR row_group IN
		select distinct
			sgm.po_group_name as group_name
		from	sa.view_ar_relationshipall_parent_valid var,
			sa_group_mapping sgm
		where	var."ObjectName" = sgm.sa_group_name and
			var."ChildObjectID" in (
				-- build the union of all group memberships of all accounts
				select	"ObjectID"
				from	sa."AR_User"
				where	lower(trim("EmailAddress")) = v_email
			) and
			sgm.po_group_name not in (
				select	g.group_name
				from	acs_rels r, groups g, membership_rels mr
				where	r.rel_id = mr.rel_id and
					r.object_id_one = g.group_id and
					mr.member_state = 'approved' and
					r.object_id_two = v_user_id
			)
	LOOP
		RAISE NOTICE 'Adding user % to group %', v_loginname, row_group.group_name;

		-- Get the PO group_id
		select group_id into v_group_id from groups where group_name = row_group.group_name;

		-- Check of the user is alrady a member of that group...
		select	count(*) into v_exists_p 
		from	acs_rels r
		where	object_id_one = v_group_id and
			object_id_two = v_user_id;

		-- Make the new user a member of the group
		IF v_exists_p = 0 THEN
			RAISE NOTICE 'Adding: user=%, group=%', v_user_id, v_group_id;
			PERFORM membership_rel__new(
				null,			-- rel_id, default null  
				'membership_rel',	-- object_type, default membership_rel
				v_group_id,		-- object_id_one
				v_user_id,		-- object_id_two
				'approved',		-- new__member_state, default approved
				null,			-- creation_user, default null
				'0.0.0.0'		-- creation_ip, default null
			);
		ELSE
			RAISE NOTICE 'User % already member of group %', v_loginname, row_group.group_name;
		END IF;

	END LOOP;

    END LOOP;
    RETURN 0;
END;$BODY$ language 'plpgsql';

-- Group the import into smaller chunks.
-- Othewise the DB will block the application...

select sa_import_user (   0,200);
select sa_import_user ( 200,200);
select sa_import_user ( 400,200);
select sa_import_user ( 600,200);
select sa_import_user ( 800,200);

select sa_import_user (1000,200);
select sa_import_user (1200,200);
select sa_import_user (1400,200);
select sa_import_user (1600,200);
select sa_import_user (1800,200);

select sa_import_user (2000,200);
select sa_import_user (2200,200);
select sa_import_user (2400,200);
select sa_import_user (2600,200);
select sa_import_user (2800,200);

select sa_import_user (3000,200);
select sa_import_user (3200,200);
select sa_import_user (3400,200);
select sa_import_user (3600,200);
select sa_import_user (3800,200);

select sa_import_user (4000,200);
select sa_import_user (4200,200);
select sa_import_user (4400,200);
select sa_import_user (4600,200);
select sa_import_user (4800,200);

select sa_import_user (5000,200);
select sa_import_user (5200,200);
select sa_import_user (5400,200);
select sa_import_user (5600,200);
select sa_import_user (5800,200);

select sa_import_user (6000,200);
select sa_import_user (6200,200);
select sa_import_user (6400,200);
select sa_import_user (6600,200);
select sa_import_user (6800,200);

